import 'dart:async';

import 'package:flutter/services.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:timezone/timezone.dart';

import 'platform_interface.dart'
    if (dart.library.io) 'platform_native.dart'
    if (dart.library.js) 'platform_web.dart';

FutureOr<DeviceInfo>? _deviceInfo;

FutureOr<DeviceInfo> get deviceInfo {
  if (_deviceInfo != null) return _deviceInfo!;
  _deviceInfo = loadPlatformInfo().then((_) {
    return _;
  });
  return _deviceInfo!;
}

Future<DeviceInfo> loadDeviceInfo() => loadPlatformInfo();

set deviceInfo(FutureOr<DeviceInfo> info) {
  assert(info is DeviceInfo, "Must provide a resolved future");
  _deviceInfo = info as DeviceInfo;
}

class SunnyInfoX implements InfoX {
  DeviceInfo? _deviceInfo;

  @override
  bool get canReadFiles => canPlatformReadFiles;

  @override
  Future<String> get currentTimeZone => currentUserTimeZone;

  @override
  Future<DeviceInfo> get deviceInfo async =>
      _deviceInfo ??= await loadDeviceInfo();

  @override
  Map<String, String> get environment => platformEnvironment;

  @override
  Future<SunnyLocalization> initializeLocalization() {
    return rootBundle
        .load('packages/timezone/data/latest.tzf')
        .then((byteData) async {
      final rawData = byteData.buffer.asUint8List();
      initializeDatabase(rawData);
      final userTimeZoneName = await currentTimeZone;
      final userLocation = getLocation(userTimeZoneName);
      final userTimeZone = userLocation.currentTimeZone;
      return SunnyLocalization(
          userTimeZone: userTimeZone, userLocation: userLocation);
    });
  }

  @override
  bool get isAndroid => isPlatformAndroid;

  @override
  bool get isIOS => isPlatformIOS;

  @override
  bool get isLinux => isPlatformLinux;

  @override
  bool get isMacOS => isPlatformMacOS;

  @override
  bool get isWeb => isPlatformWeb;

  @override
  bool get isWindows => isPlatformWindows;

  @override
  Future<DeviceInfo> loadDeviceInfo() {
    return loadPlatformInfo();
  }

  @override
  String get operatingSystem => platformName;
}
