import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:url_launcher/url_launcher.dart';
import 'app_info.dart';

Future goToAppStore(BuildContext context, AppInfo info) async {
  final url = info.appStoreUrl;
  if (url != null) {
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  } else {
    Navigator.pop(context);
  }
}

class ForcedUpgrade extends StatelessWidget {
  final AppInfo appInfo;
  final String appName;
  final Widget logoWidget;
  final Widget? backgroundWidget;

  const ForcedUpgrade(
      {Key? key,
      required this.appName,
      required this.logoWidget,
      this.backgroundWidget,
      required this.appInfo})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      child: Stack(
        fit: StackFit.expand,
        children: <Widget>[
          backgroundWidget ??
              Container(
                decoration: BoxDecoration(
                  color: sunnyColors.primaryColor,
                ),
              ),
          Column(
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              Expanded(
                flex: 2,
                child: Container(
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      Container(child: logoWidget),
                      Padding(
                        padding: const EdgeInsets.only(top: 10.0),
                      ),
                    ],
                  ),
                ),
              ),
              SizedBox(
                width: 300,
                child: PlatformCard(
                  // height: 230,
                  child: Center(
                    child: Layout.column()
                        .alignCenter
                        .crossAxisCenter
                        .padAll(20.px)
                        .spacing(sunnySpacing * 2)
                        .build([
                      Text("Please Upgrade",
                          style: sunnyText.header2,
                          textAlign: TextAlign.center),
                      Text(
                        "Your version of ${appName} is out of date. Please use the link "
                        "below to get the latest version.",
                        textAlign: TextAlign.center,
                        style: sunnyText.body1Normal,
                      ),
                      PlatformButton(
                          child: Text(
                            "Upgrade Now",
                            style: sunnyText.body2Link,
                          ),
                          onPressed: () => goToAppStore(context, appInfo))
                    ]),
                  ),
                ),
              ),
              Expanded(child: Container()),
            ],
          ),
        ],
      ),
    );
  }
}
