import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_sdk_core/services/lifecycle_aware.dart';

export 'package:sunny_sdk_core/services/lifecycle_aware.dart';

typedef LifecycleAwareBuilder<L> = L Function();
typedef LifecycleAwareContextBuilder<L> = L Function(BuildContext context);
typedef LifecycleAwareBuilder2<L, T> = L Function(T other);

typedef LifecycleAwareContextBuilder2<L, T> = L Function(
    BuildContext context, T other);
typedef LifecycleAwareBuilder3<L, A, B> = L Function(A other, B other2);
typedef LifecycleAwareBuilder4<L, A, B, C> = L Function(A a, B b, C c);
typedef LifecycleAwareContextBuilder3<L, A, B> = L Function(
    BuildContext context, A other, B other2);

Provider<E> provider<E>(LifecycleAwareBuilder<E> create,
        {bool lazy = true, Widget? child}) =>
    Provider<E>(
      create: (_) => trying(create),
      dispose: (context, final value) {
        if (value is LifecycleAware) value.dispose();
      },
      lazy: lazy,
      child: child,
    );

Provider<E> xprovider<E>(LifecycleAwareContextBuilder<E> create,
        {bool lazy = true, Widget? child}) =>
    Provider<E>(
      create: (_) => trying(() => create(_)),
      dispose: (context, final value) {
        if (value is LifecycleAware) value.dispose();
      },
      lazy: lazy,
      child: child,
    );

ProxyProvider<A, E> proxy<E, A>(LifecycleAwareBuilder2<E, A> create,
        {bool lazy = true, Widget? child}) =>
    ProxyProvider<A, E>(
      update: (c, a, s) => s ?? trying(() => create(a)),
      dispose: (context, final value) {
        if (value is LifecycleAware) {
          value.dispose();
        }
      },
      lazy: lazy,
      child: child,
    );

ProxyProvider<A, E> xproxy<E, A>(LifecycleAwareContextBuilder2<E, A> create,
        {bool lazy = true, Widget? child}) =>
    ProxyProvider<A, E>(
      update: (c, a, s) => s ?? trying(() => create(c, a)),
      dispose: (context, final value) {
        if (value is LifecycleAware) {
          value.dispose();
        }
      },
      lazy: lazy,
      child: child,
    );

ProxyProvider2<A, B, E> proxy2<E, A, B>(LifecycleAwareBuilder3<E, A, B> create,
        {bool lazy = true, Widget? child}) =>
    ProxyProvider2<A, B, E>(
      update: (c, a, b, s) => s ?? trying(() => create(a, b)),
      dispose: (context, final value) {
        if (value is LifecycleAware) value.dispose();
      },
      lazy: lazy,
      child: child,
    );

ProxyProvider3<A, B, C, E> proxy3<E, A, B, C>(
        LifecycleAwareBuilder4<E, A, B, C> create,
        {bool lazy = true,
        Widget? child}) =>
    ProxyProvider3<A, B, C, E>(
      update: (ctx, a, b, c, s) => s ?? trying(() => create(a, b, c)),
      dispose: (context, final value) {
        if (value is LifecycleAware) value.dispose();
      },
      lazy: lazy,
      child: child,
    );

ProxyProvider2<A, B, E> xproxy2<E, A, B>(
        LifecycleAwareContextBuilder3<E, A, B> create,
        {bool lazy = true,
        Widget? child}) =>
    ProxyProvider2<A, B, E>(
      update: (c, a, b, s) => s ?? trying(() => create(c, a, b)),
      dispose: (context, final value) {
        if (value is LifecycleAware) value.dispose();
      },
      lazy: lazy,
      child: child,
    );
