import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/lifecycle_main.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';

class LifecycleExtensions {}

extension BuildContextLifecycleExt on BuildContext {
  IAuthState get authState => this.get();
  bool get isLoggedIn => authState.isLoggedIn;
  bool get isNotLoggedIn => !isLoggedIn;

  DeviceInfo get deviceInfo => get();

  TextTheme get textTheme => Theme.of(this).textTheme;

  // DeviceScreenType get screenType => layoutInfo.screenType;

  LayoutInfo get layoutInfo => get();

  AppBootstrap app() => AppBootstrap.current();

  PlatformLayoutInfo get platformLayoutInfo => get();

  Stream<AuthUserProfile?> get userChanges => authState.onUserChange;

  UserDetails? get userDetails => authState.currentUser;
}

// O trying<O>(O execute(), {Logger log}) {
//   try {
//     return execute();
//   } catch (e, stack) {
//     print(e);
//     print(stack);
//     rethrow;
//   }
// }

extension UriSanitizationExt on Uri {
  String get sanitized {
    if (this.hasFragment) {
      // use the fragment as the url
      return this.fragment;
    }
    final norm = normalizePath();
    return buildString((buff) {
      buff += normalizePath();
      if (norm.hasQuery) {
        buff += "?${norm.query}";
      }
    });
  }
}

extension UriSanitizationStreamExt on Stream<Uri> {
  Stream<String> get sanitized {
    return map((url) => url.sanitized);
  }
}

extension UriSanitizationFutureExt on FutureOr<Uri> {
  FutureOr<String> get sanitized {
    return thenOr((url) => url.sanitized);
  }
}
