import 'package:firebase_auth/firebase_auth.dart' as fb;
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_screen_scaling/screen_scale_properties.dart';
import 'package:responsive_builder/responsive_builder.dart';
import 'package:sunny_core_widgets/core_ext.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_essentials/sunny_essentials.dart';
import 'package:sunny_lifecycle/modules/lifecycle_modules.dart';
import 'package:sunny_sdk_core/api/api_exceptions.dart';
import 'package:sunny_sdk_core/auth/auth_user_profile.dart';
import 'package:sunny_sdk_core/services/i_auth_state.dart';

import 'lifecycle_main.dart';

Future<P> defaultCompleteAccount<P>(BuildContext context, fb.User? profile, String accessToken) {
  return defaultLoadUserDetails(profile!) as Future<P>;
}

void defaultScreenInit(BuildContext context, LayoutInfo layoutInfo) {
  SunnyColors.init();
  SunnyColors.resolveBrightness(context);

  // final pixelRatio = WidgetsBinding.instance.window.devicePixelRatio;
  // final physicalWidth = WidgetsBinding.instance.window.physicalSize.width;
  final props = layoutInfo.screenType == DeviceScreenType.mobile
      ? ScreenScale.init(width: 375, allowFontScaling: true, allowSubpixel: true)
      : layoutInfo.screenType == DeviceScreenType.desktop
          ? ScreenScale.init(width: 375, maxWidth: 375, allowFontScaling: false, allowSubpixel: false)
          : ScreenScale.init(width: 375, maxWidth: 640, allowFontScaling: false, allowSubpixel: false);

  final textScaleFactor = props.scaleText;
  TextWidgets.lightColor = sunnyColors.g600;
  SunnyTextTheme.setFontScale(textScaleFactor);
  SunnyTextTheme.setBrightness(context);
  SunnySpacing.setScaleFactor(textScaleFactor);
  CupertinoVisualStyle.instance = CupertinoVisualStyle();
}

Future<UserDetails> defaultLoadUserDetails(fb.User? profile) {
  if (profile!.isAnonymous) {
    return SynchronousFuture(UserDetails("anonymous", profile.uid, "Anonymous", "anonymous@revelry.fm", "", "", profile.uid));
  }
  return SynchronousFuture(UserDetails(
      "firebase", profile.uid, profile.displayName!, profile.email!, profile.photoURL!, profile.phoneNumber!, profile.uid));
}

Future defaultCompleteStage1(LifecycleInit? lifecycle) {
  return SynchronousFuture(null);
}

Widget defaultBuildPlatformApp(BuildContext context, Widget app) {
  return app;
}

Future<bool> defaultSendDevicePushToken(BuildContext context, Object user, String token) {
  throw "Not implemented.  Must implement ability to send device token";
}

dynamic defaultPushMessageBodyConverter(String? type, String? body) {
  throw "Not implemented.  Must implement ability to convert push message bodies";
}

void noopBuildContext(BuildContext context) {}

Widget defaultBuildOuterContext(Widget inner) => inner;

class AnonymousAuthModule implements AuthModule<Object> {
  static final anonymousUser = 'anonymous-user';
  static final userDetails =
      UserDetails(anonymousUser, anonymousUser, 'Anonymous User', 'anonymous@sunny', '', '', anonymousUser);

  static final anonAuthUserProfile = AuthUserProfile(null, AnonymousAuthModule.userDetails, AuthEventSource.framework);

  const AnonymousAuthModule();

  @override
  Future<AuthUserProfile?> getInitialUser() {
    return SynchronousFuture(anonAuthUserProfile);
  }

  @override
  IAuthState initializeState(AuthUserProfile? profile, Stream<ErrorStack> appErrors) {
    return AnonymousAuthState();
  }

  @override
  Future<UserDetails> loadProfile(Object user) {
    return SynchronousFuture(userDetails);
  }
}

class AnonymousAuthState implements IAuthState {
  @override
  String? get accountId => AnonymousAuthModule.anonymousUser;

  @override
  AuthUserProfile? get current => AuthUserProfile(null, AnonymousAuthModule.userDetails, AuthEventSource.framework);

  @override
  UserDetails? get currentUser => AnonymousAuthModule.userDetails;

  @override
  bool get isLoggedIn => true;

  @override
  bool get isNotLoggedIn => false;

  @override
  void onPreLogout(Future<bool> Function() onPreLogout) {}

  @override
  Stream<AuthUserProfile> get userStateStream =>
      Stream.value(AuthUserProfile(null, AnonymousAuthModule.userDetails, AuthEventSource.framework));

  @override
  Stream<AuthUserProfile?> get onUserChange => userStateStream;
}
