import 'package:firebase_auth/firebase_auth.dart' as fb;
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:logging/logging.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_essentials/sunny_essentials.dart';
import 'package:sunny_fluro/sunny_fluro.dart';

import 'login_provider.dart';

class EmailPasswordLoginProvider extends LoginProvider {
  static const routePath = "/loginEmailPassword";
  static const registerRoutePath = "/registerEmailPassword";
  final Future Function(BuildContext context)? onWeb;

  const EmailPasswordLoginProvider({this.onWeb});

  final String name = "email";
  static final _log = Logger("emailLoginHandler");

  @override
  Widget logo(double size, {Color color = Colors.white}) {
    return Icon(
      Icons.email,
      color: color,
      size: size,
    );
  }

  @override
  bool get isEnabled => true;

  @override
  Future signOut() async {
    /// This plugin doesn't cache anything, so nothing to do.
  }

  @override
  Future<AuthResponse> doAttemptRegister(BuildContext context,
      {credentials}) async {
    try {
      try {
        final router = context.get<FRouter>();
        final route = router.matchRoute(registerRoutePath)!;
        assert(!route.isMissing,
            "You must provide a route /loginEmailPassword to use this login handler");

        final usernamePassword = await route.route!
            .openModal(context, expand: false, nestModals: true);
        if (usernamePassword == null) {
          return AuthResponse.cancelled();
        }
        assert(usernamePassword is UserCredential,
            "The response from /loginEmailPassword must be UserCredential");
        var c = usernamePassword as fb.UserCredential;
        return c.user == null
            ? AuthResponse.cancelled()
            : AuthResponse.userLogin(user: c.user);
      } catch (e, stack) {
        _log.severe("Error from server: $e", e, stack);
        return AuthResponse.error(e);
      }
    } catch (e, stack) {
      _log.severe("Error from server: $e", e, stack);
      return AuthResponse.error("There was an error logging you in");
    }
  }

  @override
  Future<AuthResponse> doAttemptLogin(BuildContext context) async {
    try {
      try {
        final router = context.get<FRouter>();
        final route = router.matchRoute(routePath)!;
        assert(!route.isMissing,
            "You must provide a route /loginEmailPassword to use this login handler");

        final usernamePassword = await route.route!
            .openModal(context, expand: false, nestModals: true);
        if (usernamePassword == null) {
          return AuthResponse.cancelled();
        }
        assert(usernamePassword is UserCredential,
            "The response from /loginEmailPassword must be UserCredential");
        var c = usernamePassword as fb.UserCredential;
        return c.user == null
            ? AuthResponse.cancelled()
            : AuthResponse.userLogin(user: c.user);
      } catch (e, stack) {
        _log.severe("Error from server: $e", e, stack);
        return AuthResponse.error(e);
      }
    } catch (e, stack) {
      _log.severe("Error from server: $e", e, stack);
      return AuthResponse.error("There was an error logging you in");
    }
  }
}

class EmailPasswordCredentials {
  String? email;
  String? password;

  EmailPasswordCredentials(
      {required String this.email, required String this.password});

  EmailPasswordCredentials.empty()
      : email = null,
        password = null;
}
