import 'package:firebase_auth/firebase_auth.dart' as fb;
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:logging/logging.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_sdk_core/api/third_party_creds.dart';

import 'login_provider.dart';

final _log = Logger("googleLoginProvider");

class GoogleLoginProvider extends LoginProvider {
  const GoogleLoginProvider();

  final String name = "google";

  String get clientId =>
      ThirdPartyCreds.googleSignIn.credentials.clientId ??
      (throw Exception("You didn't set the google clientId propertly"));

  @override
  bool get isEnabled => true;

  @override
  Future<AuthResponse> doAttemptLogin(BuildContext context,
      {credentials}) async {
    try {
      final GoogleSignInAccount? googleUser = await GoogleSignIn().signIn();
      _log.info("Got response from sign in: ${googleUser?.id}");
      if (googleUser == null) {
        return AuthResponse.cancelled();
      }

      final GoogleSignInAuthentication googleAuth =
          await googleUser.authentication.timeout(3.seconds);

      _log.info("Waited for authentication (with timeout): ${googleUser.id}");
      final credential = fb.GoogleAuthProvider.credential(
        accessToken: googleAuth.accessToken,
        idToken: googleAuth.idToken,
      );

      _log.info("Got firebase creds: ${googleUser.id}");

      return AuthResponse.success(credential);
    } catch (e, stack) {
      _log.severe("Error with google sign in: $e", e, stack);
      rethrow;
    }
  }

  @override
  Future signOut() {
    return GoogleSignIn(clientId: this.clientId).disconnect();
  }
}
