import 'package:dartx/dartx.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:sunny_dart/helpers.dart';
import 'package:sunny_lifecycle/login/login_provider.dart';
import 'package:sunny_lifecycle/modules/lifecycle_modules.dart';
import 'package:sunny_sdk_core/api/api_exceptions.dart';
import 'package:sunny_sdk_core/auth/auth_user_profile.dart';
import 'package:sunny_sdk_core/auth/firebase_api_auth.dart';
import 'package:sunny_sdk_core/services/i_auth_state.dart';
import 'package:sunny_sdk_core/services/resolver_inits.dart';

import '../auth_state.dart';

class FirebaseAuthAppModule
    with LoggingMixin
    implements AuthModule<User>, RegisteringModule {
  final Future<UserDetails> Function(User fbUser) loadUserProfile;
  final AccountCompleter<UserDetails>? accountCompleter;
  final List<LoginProvider> loginProviders;

  FirebaseAuthState? state;

  FirebaseAuthAppModule(
      {required this.loadUserProfile,
      this.accountCompleter,
      List<LoginProvider>? loginProviders})
      : loginProviders = loginProviders ??
            [
              LoginProvider.apple,
              // LoginProvider.facebook,
              LoginProvider.google,
            ];

  Future<UserDetails> loadProfile(User user) async {
    return await this.loadUserProfile(user);
  }

  @override
  Future<AuthUserProfile?> getInitialUser() async {
    var initUser = await FirebaseAuth.instance
        .authStateChanges()
        .first
        .timeout(1.seconds, onTimeout: () {
      log.info("Timed out after 1 second waiting for initial firebase auth");
      return null;
    });

    /// This takes a few seconds, but we do it way up here so we can sit on
    /// the landing screen longer.
    UserDetails? initProfile;
    if (initUser != null && !initUser.isAnonymous) {
      FirebaseApiAuth.user = initUser;
      try {
        //AccountsApi(apiClient).profile();
        initProfile = await loadUserProfile(initUser);
      } on ApiResponseException catch (e) {
        if (e.statusCode == kPreconditionRequired ||
            e.statusCode == kPreconditionFailed ||
            e.statusCode == kUnauthorized ||
            e.statusCode == kForbidden) {
          /// Leave this alone.  It just means that we need to
          /// finish account creation
          log.warning(
              "Found a firebase token, but couldn't authenticate against the server with it");
          await FirebaseAuth.instance.signOut();
          // Even though we appear to be authenticated, let's clear the state
          initUser = null;
          FirebaseApiAuth.user = null;
        } else {
          rethrow;
        }
      }
    }
    return AuthUserProfile(initUser, initProfile, AuthEventSource.initial);
  }

  @override
  List<Inst> registrations() {
    return [
      Inst<IAuthState>.constant(state!),
      Inst<FirebaseAuthState>.constant(state!),
    ];
  }

  @override
  IAuthState initializeState(
      AuthUserProfile? profile, Stream<ErrorStack> appErrors) {
    state ??= FirebaseAuthState(this, profile?.fbUser, profile?.profile,
        appErrors: appErrors);
    return state!;
  }
}
