import 'dart:async';

import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/sunny_lifecycle.dart';

import '../lifecycle_main.dart';

AppRouteMatch rootAppRouteMatch([Widget splashPage()?]) => AppRouteMatch.ofMap(
      AppPageRoute('/', (context, _) => splashPage?.call() ?? emptyBox, (_) => const DefaultRouteParams()),
      const {},
    );

class FRouterInformationParser with LoggingMixin implements RouteInformationParser<AppRouteMatch> {
  final FRouter router;
  final AppBootstrap app;

  const FRouterInformationParser(this.router, this.app);

  String get loggerName => "routeParser";

  @override
  Future<AppRouteMatch> parseRouteInformation(RouteInformation routeInformation) async {
    final matched = router.matchRoute(routeInformation.location);
    if (matched == null) {
      log.info("parsed: [route=${routeInformation.location}]");
      return rootAppRouteMatch(app.splashPageWidget);
    } else {
      log.info("parsed: [route=${routeInformation.location}] => $matched");
    }

    return matched;
  }

  @override
  RouteInformation restoreRouteInformation(AppRouteMatch configuration) {
    var routeUri = configuration.route?.routeUri(configuration.parameters);

    log.info("restore $routeUri");
    if (routeUri == null) {
      return RouteInformation(location: '/');
    } else {
      return RouteInformation(
        state: configuration.parameters,
        location: routeUri,
      );
    }
  }
}
