import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:flutter/cupertino.dart';
import 'package:logging/logging.dart';
import 'package:sunny_dart/helpers.dart';
import 'package:sunny_dart/helpers/logging_mixin.dart';

import 'fade_animation_page.dart';
import 'frouter_delegate.dart';

abstract class FRouterNavigatorObserver extends NavigatorObserver
    with LoggingMixin {
  void doPop(RouteInformation? information);
  void doPush(RouteInformation? information);

  @override
  void didPush(Route<dynamic> route, Route<dynamic>? previousRoute) {
    doPush(_toRouteInfo(route,
        "Did push ${route.settings} (previous ${previousRoute?.settings})"));
  }

  @override
  void didPop(Route<dynamic> route, Route<dynamic>? previousRoute) {
    doPop(_toRouteInfo(route, "Overlay pop ${route}, previous $previousRoute"));
  }

  @override
  void didRemove(Route<dynamic> route, Route<dynamic>? previousRoute) {
    log.warning("We're not equipped for this: ");
    log.warning("  $route");
    log.warning("  $previousRoute");
    // report(previousRoute,
    //     "Did remove ${route?.settings} (previous ${previousRoute?.settings})");
  }

  @override
  void didReplace({Route<dynamic>? newRoute, Route<dynamic>? oldRoute}) {
    var src = "Did replace ${newRoute?.settings} (old ${oldRoute?.settings})";
    doPop(_toRouteInfo(oldRoute, src));
    doPush(_toRouteInfo(newRoute, src));
  }

  RouteInformation? _toRouteInfo(Route? route, String source) {
    final settings = route?.settings;
    if (settings is PathRouteSettings && settings.resolvedPathOrName != null) {
      _log.fine(
          "${this.runtimeType}[overlay=${settings.route};pathRoute=true]; reason: $source");

      return settings;
    } else if (settings is FadeAnimationPage) {
      _log.fine(
          "${this.runtimeType}[overlay=${settings.route}]; reason $source");

      return null;
    } else {
      _log.fine("${this.runtimeType}[overlay=anon] $source");
      return RouteInformation(location: "/${uuid()}", state: source);
    }
  }
}

class RootNavigatorObserver extends FRouterNavigatorObserver {
  final FRouterDelegate delegate;

  RootNavigatorObserver(this.delegate);

  @override
  void doPop(RouteInformation? information) =>
      delegate.reportRootPop(information);

  @override
  void doPush(RouteInformation? information) =>
      delegate.reportRootPush(information);
}

/// Passes events from a child router up to the root.  It's assumed that
/// the root provider will not receive these events without this connector
/// and that it should only report the url out to the navigator
class TabRouteObserver extends FRouterNavigatorObserver {
  final FRouterDelegate delegate;

  /// The  index of the tab sending the events
  final int tabIndex;

  TabRouteObserver(this.delegate, {required this.tabIndex});

  @override
  void doPop(RouteInformation? information) =>
      delegate.reportTabPop(information);

  @override
  void doPush(RouteInformation? information) =>
      delegate.reportTabPush(information);
}

extension PathRouteSettingsHelperExt on PathRouteSettings {
  String? get resolvedPathOrName => resolvedPath ?? this.route;
}

final _log = Logger("routes");

extension RouteToInformation on Route {}
