import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';
import 'package:sunny_essentials/state/state_observer.dart';
import 'package:sunny_lifecycle/modules/lifecycle_modules.dart';

import 'frouter_delegate.dart';
import 'routing_system.dart';

class RoutingPortal extends StatefulWidget {
  final RoutingModule routing;

  const RoutingPortal({Key? key, required this.routing}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _RoutingPortalState();
  }
}

class _RoutingPortalState extends BaseState<RoutingPortal> {
  late RoutingSystem _system;

  @override
  void initState() {
    super.initState();
    var parentRouter = SunnyRouter.tryDelegateOf(context);
    _system = widget.routing.createRoutingSystem(parentRouter);
  }

  @override
  void dispose() {
    _system.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Provider<SunnyRouter>.value(
      value: SunnyRouter(_system.delegate as FRouterDelegate),
      builder: (context, _) => Router(
        routeInformationProvider: _system.provider,
        backButtonDispatcher: _system.backButtonDispatcher,
        routeInformationParser: _system.parser,
        routerDelegate: _system.delegate,
      ),
    );
  }
}
