import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:logging/logging.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'frouter_state.dart';
import 'route_observer.dart';

class FRouterRootProviderPlatform extends PlatformRouteInformationProvider {
  static final log = Logger("frouter");

  final FRouteState state;
  FRouterRootProviderPlatform(
      {RouteInformation? initialRoute, required this.state})
      : super(initialRouteInformation: initialRoute);

  /// Called when the host tells the application to push a new route onto the
  /// navigator.
  ///
  /// On web, this is called when the user changes the url manually, or clicks
  /// the back or forward buttons
  @override
  Future<bool> didPushRoute(String route) {
    log.info("platform:didPushRoute: $route");
    return handleNewRoute(route);
  }

  /// Called when the host tells the application to push a new route onto the
  /// navigator.
  ///
  /// On web, this is called when the user changes the url manually, or clicks
  /// the back or forward buttons
  @override
  Future<bool> didPushRouteInformation(RouteInformation routeInformation) {
    log.info(
        "platform:didPushRouteInformation: ${routeInformation.location} state: ${routeInformation.state}");
    return handleNewRoute(routeInformation.location);
  }

  /// This is called when the router says there's a new route - and we should report
  /// that route back to the browser/OS
  @override
  void routerReportsNewRouteInformation(RouteInformation routeInformation) {
    log.info(
        "platform:didPushRouteInformation: ${routeInformation.location} ${routeInformation.runtimeType}");
    return super.routerReportsNewRouteInformation(routeInformation);
  }

  /// Overrides the root behavior to allow us to interpret browser route pushes
  /// that should be treated as pop instead of push.
  ///
  /// If we handle the method here, it will not be passed on to the delegate
  ///
  /// Maybe this should move to delegate?
  Future<bool> handleNewRoute(String? route) {
    // Look backwards in history for a match, then pop routes to match.
    // if (route == null) return SynchronousFuture(false);
    // var matches = state.matchHistory(route);
    // if (matches.isNotEmpty) {
    //   // Pop the route
    //   for (var match in matches) {
    //     if (match.tab != null) {
    //       if (match.tab != state.currTab) {
    //         log.info(
    //             "platform:didPushRouteInformation > move tab ${match.tab}");
    //         state.currTab = match.tab;
    //       } else {
    //         var tabNavState =
    //             state.tabState[match.tab!].navigatorKey.currentState!;
    //         if (tabNavState.canPop() == true) {
    //           tabNavState.popUntil((route) {
    //             var isPath = route.path == route;
    //             if (!isPath) {
    //               log.info(
    //                   "platform:didPushRouteInformation ^ popped ${match.route}");
    //             }
    //             return isPath;
    //           });
    //         } else {
    //           log.warning(
    //               "platform:didPushRouteInformation ! unable to pop ${match.tab}");
    //         }
    //       }
    //     } else {
    //       //pop root?
    //       log.warning("platform:didPushRouteInformation can't pop root (TBD)");
    //     }
    //   }
    //
    //   return SynchronousFuture(true);
    // } else {
    //   /// How do we know where to push this route?
    //
    //   log.warning(() {
    //     var str = "";
    //     str +=
    //         'platform:didPushRouteInformation No found routes for ${route}\n';
    //     str += '  in ->\n';
    //     if (state.history.isEmpty) {
    //       str += '    - no history\n';
    //     } else {
    //       str +=
    //           '    - ${state.history.map((element) => element.route).join('\n    - ')}';
    //     }
    //     return str;
    //   });
    // }
    return super.didPushRoute(route!);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void removeListener(VoidCallback listener) {
    super.removeListener(listener);
  }

  @override
  void addListener(VoidCallback listener) {
    super.addListener(listener);
  }

  @override
  RouteInformation? get value {
    return super.value;
  }
}

extension RoutePathExt on Route {
  String? get path {
    final self = this.settings;
    if (self is PathRouteSettings) {
      return self.resolvedPathOrName;
    } else {
      return self.name;
    }
  }
}
