import 'package:equatable/equatable.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:sunny_core_widgets/core_ext.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:sunny_lifecycle/routing/route_history.dart';
import 'package:sunny_lifecycle/routing2/frouter_parser.dart';
import 'package:sunny_sdk_core/auth/auth_user_profile.dart';

class FRouteState extends ChangeNotifier
    with ThrottledChangeNotifier, EquatableMixin, LoggingMixin {
  FRouteState(
      {required List<HistoryObserver> historyObservers,
      AuthUserProfile? loggedIn,
      bool? isInitted})
      : _history = RouteHistory(observers: historyObservers),
        _isInitted = false;

  final RouteHistory _history;

  bool _isInitted;

  bool get isInitted => _isInitted;

  String? _focusedChild;

  String? get focusedChild => _focusedChild;

  set focusedChild(String? focusedChild) {
    if (focusedChild != _focusedChild) {
      _focusedChild = focusedChild;
      notifyListeners();
    }
  }

  List<PendingRoute> _viewStack = [];

  Iterable<PendingRoute> get viewStack => _viewStack;

  void pushHistoryItem(FRouteMatch item) {
    _history.push(RouteHistoryItem(item.match.resolvedUri!, item.matchedBy.path,
        isPage: true));
  }

  // set rootRoutes(Iterable<PendingRoute> rootRoutes) {
  //   final pendingList = rootRoutes.toList();
  //   if (_rootRoutes == pendingList) {
  //     return;
  //   }
  //
  //   _rootRoutes = pendingList;
  //   notifyWithLimit();
  // }

  @override
  String toString() {
    return joinString((str) {
      str += "FRouteState{isInit:${_isInitted}";
    }, ';');
  }

  @override
  List<Object?> get props => [
        _isInitted,
        _viewStack,
      ];

  bool hasRoutes() {
    return _viewStack.isNotEmpty;
  }

  bool popRoute({bool notify = true}) {
    _viewStack.removeLast();
    if (notify) notifyWithLimit();
    return _viewStack.isNotEmpty;
  }

  bool pushRoute(AppRouteMatch? match,
      {bool isPage = true, required bool replace, bool notify = true}) {
    if (match == null) return false;
    final route = PendingRoute(match, isPage: isPage);

    if (replace) {
      _viewStack.removeLast();
    }
    _viewStack.add(route);
    if (notify) notifyWithLimit();
    return true;
  }

  void popHistory() {
    _history.pop();
  }

  List<RouteHistoryItem> matchHistory(String route) {
    return [];
  }
}

const appValueKey = ValueKey<String>('/app');
const rootValueKey = ValueKey<String>('/');
const splashValueKey = ValueKey<String>('/splash');
const loginValueKey = ValueKey<String>('/login');

final rootKeys = [
  rootValueKey,
  appValueKey,
  splashValueKey,
  loginValueKey,
].map((_) => _.value).toSet();

extension AppRouteMatchUriBuilderExt on AppRouteMatch {
  String? get resolvedUri {
    return this.route?.routeUri(parameters);
  }
}

class RouteHistory extends ListDelegateMixin<RouteHistoryItem>
    with LoggingMixin {
  final items = <RouteHistoryItem>[];
  final List<HistoryObserver> observers;

  RouteHistory({required this.observers});

  void push(RouteHistoryItem item) {
    items.insert(0, item);
    for (var o in observers) {
      o.didPushHistory(item, [...items]);
    }
  }

  void pop() {
    var popped = items.removeAt(0);
    for (var o in observers) {
      o.didPopHistory(popped, [...items]);
    }
  }

  List<RouteHistoryItem> match(String route) {
    var results = <RouteHistoryItem>[];
    var found = false;
    //todo: how far back to look?
    for (var item in items.skip(1).take(3)) {
      found = item.route == route;
      if (!found && item.isPage) {
        return [];
      }
      results.add(item);
      if (found) {
        return results;
      }
    }
    return [];
  }

  @override
  List<RouteHistoryItem> get delegate => items;
}

class PendingRoute extends Equatable {
  final AppRouteMatch? match;
  final bool isPage;

  PendingRoute(this.match, {this.isPage = true});

  @override
  List<Object> get props => [route, isPage];

  String get route =>
      match!.resolvedUri ?? illegalState("Must provide route uri");
}

extension IterablePagesExt on Iterable<PendingRoute> {
  Iterable<PendingRoute> get pages => this.where((p) => p.isPage);
}

extension PendingRouteToHistory on PendingRoute {
  RouteHistoryItem toHistory({String? childPath}) =>
      RouteHistoryItem(this.route, childPath, isPage: isPage);
}
