import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';
import 'package:overlay_support/overlay_support.dart';
import 'package:provider/provider.dart';
import 'package:sunny_core_widgets/intl/sunny_intl.dart';
import 'package:sunny_core_widgets/routes.dart';
import 'package:sunny_dart/sunny_get.dart';
import 'package:sunny_essentials/provided.dart';
import 'package:sunny_essentials/state.dart';
import 'package:sunny_essentials/theme.dart';
import 'package:sunny_lifecycle/app_info.dart';
import 'package:sunny_lifecycle/modules/lifecycle_modules.dart';
import 'package:sunny_lifecycle/push_notification_service.dart';
import 'package:sunny_lifecycle/routing2/routing_system.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';

import 'lifecycle_main.dart';

class SunnyApp extends StatefulWidget {
  final AppInfo appInfo;
  final AppBootstrap app;

  const SunnyApp(this.appInfo, {required this.app, Key? key}) : super(key: key);

  @override
  _SunnyAppState createState() => _SunnyAppState();
}

class _SunnyAppState extends BaseState<SunnyApp> {
  // late FRouteState routeState;
  late RoutingSystem _routing;

  PushNotificationService? _push;

  @override
  void initState() {
    super.initState();

    _push = app.pushNotifications?.initializePushNotifications(() => this.mounted ? context : null, widget.appInfo);

    if (_push != null) {
      final res = sunny.get<BuildContextResolver>(context: context);
      res.registerSingleton(context, _push!);
    }
    app.initializeFromWithin(context);

    // var initialRoute = RouteInformation(
    //   location: "/splash",
    //   state: routeState,
    // );
    // _platformProvider = PlatformRouteInformationProvider(
    //   initialRouteInformation: initialRoute,
    // );

    this._routing = app.routing.createRoutingSystem();
  }

  @override
  void dispose() {
    _routing.dispose();

    // _platformProvider.dispose();
    super.dispose();
  }

  AppBootstrap get app => widget.app;

  @override
  Widget build(BuildContext context) {
    /// Set up all the scaling business

    final brightness = sunny.get<Brightness>(context: context);
    final layoutInfo = context.layoutInfo;
    app.screenInit.call(context, layoutInfo);

    final themes = sunny.get<Themes>(context: context);

    return MultiProvider(
      providers: [
        Provider.value(value: _push),
      ],
      child: OverlaySupport(
        key: Key("outerOverlaySupport"),
        child: PlatformProvider(
          settings: PlatformSettingsData(
            platformStyle: PlatformStyleData(
              web: layoutInfo.platformStyle,
            ),
          ),
          builder: (context) {
            return app.serviceResolver.register(
              context,
              [
                // Inst.constant(
                //     const SunnyFormTheme(starForRequiredFields: false)),
                for (var module in app.modules.whereType<RegisteringModule>())
                  for (var inst in module.registrations()) inst,
                Inst.factory((_) => SunnyIntl.load(enUs)),
                Inst.constant(_routing),
              ],
              key: Key("root-context-$brightness"),
              child: Builder(
                builder: (context) => app.buildPlatformApp(
                  context,
                  PlatformApp.router(
                    debugShowCheckedModeBanner: false,
                    title: app.appTitle,
                    routeInformationParser: _routing.parser,
                    // routeInformationProvider: _routing.provider,

                    /// The router doesn't listen to the delegate
                    routerDelegate: _routing.delegate,
                    backButtonDispatcher: _routing.backButtonDispatcher,
                    // routeInformationProvider: _routeProvider,
                    // onGenerateRoute: app.router?.onGenerateRoute,
                    // initialRoute: "/",
                    localizationsDelegates: [
                      DefaultMaterialLocalizations.delegate,
                    ],
                    cupertino: (context, platform) => CupertinoAppRouterData(
                      theme: context.sub<CupertinoThemeData>(),
                      // color: TrippiColors.primaryColor,
                      color: sunnyColors.primaryColor,
                      title: app.appTitle,
                    ),
                    material: (context, platform) => MaterialAppRouterData(
                      theme: themes.lightTheme,
                      color: themes.lightTheme.primaryColor,
                      darkTheme: app.isChangeBrightness ? themes.darkTheme : themes.lightTheme,
                    ),
                  ),
                ),
              ),
            );
          },
        ),
      ),
    );
  }
}
