import 'dart:core';
import 'dart:io';

main() async {
  final outputDir = Directory("assets");

  print("With asset directory: $outputDir");
  File assetsOutput = File("lib/assets.dart");
  if (assetsOutput.existsSync()) {
    assetsOutput.deleteSync();
  }
  var assetsDart = "";
  outputDir
      .listSync(recursive: true, followLinks: false)
      .whereType<Directory>()
      .map((d) => d.absolute)
      .forEach((directory) {
    final prefix = directory.pathFromAssets;
    final log = (str) => print("[$prefix] $str");

    final p = outputDir.uri.resolveUri(directory.uri);
    log("Processing $p");
    var str = "\n\nclass ${directory.fileName} {\n\n";

    directory
        .listSync(recursive: false, followLinks: false)
        .whereType<File>()
        .map((f) => f.absolute)
        .forEach((entity) {
      final variant = entity.variant(log);
      if (variant?.isEmpty == false) {
        // move to folder
        final xDir = entity.parent.dir("$variant.0x");
        if (!xDir.existsSync()) {
          log("Creating ${xDir.path}");
          xDir.createSync();
        }

        String baseName = entity.baseNameNoVariant;
        if (baseName.contains(" ") ||
            directory.pathFromAssets.startsWith("images")) {
          log("Convert to lower_case: ${baseName}");
          baseName = baseName.replaceAll(" ", "_").toLowerCase();
        }

        log("Move $baseName.${entity.extension} to $variant.0x/ folder");
        entity.renameSync(
            entity.parent + "$variant.0x/$baseName.${entity.extension}");
      } else {
        if (entity.fileName.contains(" ") ||
            directory.pathFromAssets.startsWith("images")) {
          final name = entity.fileName;
          final rename = name.replaceAll(" ", "_").toLowerCase();
          if (name != rename) {
            log("Renaming file $name > $rename ${entity.parent.file(rename)}");
            entity.renameSync(entity.parent + rename);
          }
        }
      }

      str +=
          "  static const ${entity.fileName.sanitize()} = 'assets/${entity.pathFromAssets}';\n";
    });
    str += "\n}\n\n\n";

    if (!directory.fileName.startsWith(RegExp("[0-9]\.0x"))) {
      assetsDart += str;
    }
  });

  assetsOutput.writeAsStringSync(assetsDart, flush: true);
}

extension StringExt on String {
  String sanitize() => this.replaceAll("-", "_").replaceAll("\.", "_");
  String operator *(int x) {
    var s = '';
    for (var i = 0; i < x; i++) {
      s += this;
    }
    return s;
  }
}

extension NumExt on num {
  String get indent => " " * this.toInt();
}

extension DirectoryExt on Directory {
  bool hasChild(String name) {
    return file(name).existsSync();
  }

  File file(String name) {
    return File(this + name);
  }

  Directory dir(String name) {
    return Directory(this + name);
  }

  String operator +(String name) {
    return "${this.path}/$name";
  }
}

extension FileExt on File {
  String? variant(void log(String str)) {
    final baseName = this.baseName;
    log("Checking baseName $baseName");
    if (baseName.contains("\@") && baseName.endsWith("x")) {
      final res = baseName.substring(baseName.length - 2, baseName.length - 1);
      log("Found resolution: $res");
      return res;
    }
    return null;
  }
}

extension FileSystemEntityExt on FileSystemEntity {
  File variant(String variant) {
    return parent.file("$baseName$variant.$extension");
  }

  String get pathFromAssets => path.replaceAll(RegExp(".*\/assets\/"), "");

  File renamed(String name) {
    return parent.file(name);
  }

  String get extension {
    return nameParts[1];
  }

  List<String> get nameParts {
    final fileName = this.fileName;
    final ll = fileName.lastIndexOf("\.");
    if (ll == -1) {
      return [fileName, ''];
    } else {
      final baseName = fileName.substring(0, ll);
      final extension = fileName.substring(ll + 1, fileName.length);
      return [baseName, extension];
    }
  }

  String get baseName => nameParts[0];

  String get baseNameNoVariant {
    final fileName = this.fileName.replaceAll(RegExp("\@.x"), "");
    print("No variant: $fileName");
    if (!fileName.contains("\.")) {
      return "";
    }

    return fileName.split("\.").first;
  }

  String get fileName => path.split("/").last;
}
